<?php
/**
 * Plugin Name: Woo PagosNet
 * Plugin URI: http://www.catarco.com
 * Description: Pagos en Efectivo o con Tarjeta de crédito/débito usando PagosNet de Bolivia.
 * Author: Camilo Camargo
 * Author URI: http://www.catarco.com
 * Version: 1.0.0
 * Text Domain: wc-pagosnet
 *
 * Copyright: (c) Camio Camargo, (camilo@catarco.com) and WooCommerce
 *
 *
 * @author    Camilo Camargo
 * @copyright Copyright: (c) Camio Camargo, (camilo@catarco.com) and WooCommerce
 *
 */
 
defined( 'ABSPATH' ) or exit;

define( 'WOO_DIR', plugin_dir_path( __FILE__ )); 

// Comprobar si Woocommerce esta activo en el sitio
if ( ! in_array( 'woocommerce/woocommerce.php', apply_filters( 'active_plugins', get_option( 'active_plugins' ) ) ) ) {
	return;
}

require_once WOO_DIR . 'includes/Autoloap.php';


/**
 * Agregar la pasarela de pago a Woocommerce
 * 
 * @since 1.0.0
 * @param array $gateways todas las pasarelas disponibles 
 * @return array $gateways todas las gateways + gateways PagosNet Efectivo
 */
function woo_pagosnet_efectivo_add_to_gateways( $gateways ) {
	$gateways[] = 'Woo_Pagosnet_efectivo';
	$gateways[] = 'Woo_Pagosnet_tarjeta';
	return $gateways;
}

add_filter( 'woocommerce_payment_gateways', 'woo_pagosnet_efectivo_add_to_gateways' );


/**
 * Pagos en efectivo con PagosNet
 *
 * Realizar pagos en Woocommerce con PagosNet en efectivo
 * 
 *
 * @class 		WC_Gateway_Offline
 * @extends		WC_Payment_Gateway
 * @version		1.0.0
 * @package		WooCommerce/Classes/Payment
 * @author 		SkyVerge
 */
add_action( 'plugins_loaded', 'woo_pagosnet_efectivo_gateway_init', 11 );
add_action( 'plugins_loaded', 'woo_pagosnet_tarjeta_gateway_init', 11 );

function woo_pagosnet_efectivo_gateway_init() {

	class Woo_Pagosnet_efectivo extends WC_Payment_Gateway {

		public $pagosNet = null;
		public $codigoRecaudacion = null;

		/**
		 * Constructor for the gateway.
		 */
		public function __construct() {
	  
			$this->id                 = 'woo_pagosnet_efectivo';
			$this->icon               = plugins_url( 'woo-pagosnet/assets/img/logo.png', plugin_dir_path( __FILE__ ) );
			$this->has_fields         = false;
			$this->method_title       = __( 'PagosNet en Efectivo', 'woo_pagosnet_efectivo' );
			$this->method_description = __( 'Realizar pagos en efectivo usando PagosNet', 'woo_pagosnet_efectivo' );
		  
			// Cargar configuraciones.
			$this->init_form_fields();
			$this->init_settings();
		  
			// Definir variables
			$this->title        = $this->get_option( 'title' );
			$this->description  = $this->get_option( 'description' );
			$this->usuario = $this->get_option( 'usuario');
			$this->pass = $this->get_option( 'pass');
			$this->url_pagosnet = $this->get_option( 'url_pagosnet');
		  
			// Actions
			add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, array( $this, 'process_admin_options' ) );
		  
			// Customer Emails
			add_action( 'woocommerce_email_before_order_table', array( $this, 'email_instructions' ), 10, 3 );
		
			//Texto de la orden
			add_filter('woocommerce_thankyou_order_received_text', array($this, 'order_received_text'), 10, 2 );
		
		}
	
		/**
		 * Initialize Gateway Settings Form Fields
		 */
		public function init_form_fields() {
	  
			$this->form_fields = apply_filters( 'wc_offline_form_fields', array(

				'url-api' => array(
					'title'       => __( 'Proporcione esta información a PagosNet para notificar los pagos en Efectivo', 'woo_pagosnet_efectivo' ),
					'type'        => 'title',
					'description' => sprintf( __( '<code>'. plugins_url() . '/woo-pagosnet/api/public/registrar/transaccion' . '</code>', 'woo_pagosnet_efectivo' )),
				),

		  
				'enabled' => array(
					'title'   => __( 'Activar/Desactivar', 'woo_pagosnet_efectivo' ),
					'type'    => 'checkbox',
					'label'   => __( 'Activar los pagos en efectivo', 'woo_pagosnet_efectivo' ),
					'default' => 'yes'
				),
				
				'title' => array(
					'title'       => __( 'Título', 'woo_pagosnet_efectivo' ),
					'type'        => 'text',
					'description' => __( 'Título a mostrar' ),
					'default'     => __( 'Pagos en efectivo con PagosNet', 'woo_pagosnet_efectivo' ),
					'desc_tip'    => true,
				),
				'usuario' => array(
					'title'       => __( 'Usuario WS PagosNet', 'woo_pagosnet_efectivo' ),
					'type'        => 'text',
					'description' => __( 'Usuario proporcionado por PagosNet para el comercio' ),
					'default'     => '',
					'desc_tip'    => true,
				),
				'pass' => array(
					'title'       => __( 'Contraseña WS PagosNet', 'woo_pagosnet_efectivo' ),
					'type'        => 'text',
					'description' => __( 'Contraseña proporcionada por PagosNet para el comercio' ),
					'default'     => __( 'Contraseña', 'woo_pagosnet_efectivo' ),
					'desc_tip'    => true,
				),
				'url_pagosnet' => array(
					'title'       => __( 'URL de preproducción/producción', 'woo_pagosnet_efectivo' ),
					'type'        => 'text',
					'description' => __( 'URL proporcionada por PagosNet para modo pruebas o producción.' ),
					'default'     => __( 'Usuario', 'woo_pagosnet_efectivo' ),
					'desc_tip'    => true,
				),
				
				'description' => array(
					'title'       => __( 'Descripción', 'woo_pagosnet_efectivo' ),
					'type'        => 'text',
					'description' => __( 'Descripción para el método de pago' ),
					'default'     => __( 'Realice sus Pagos en Efectivo mediante PagosNet. Al pulsar en Realizar Pedido, se generará su código de recaudación.', 'woo_pagosnet_efectivo' ),
					'desc_tip'    => true,
				),

				'desc-datos-api' => array(
					'title'       => __( 'Credenciales para la API', 'woo_pagosnet_efectivo' ),
					'type'        => 'title',
					'description' => sprintf( __( '<span style="font-weight:bold;">Genere las credenciales  en: WooCommerce > Ajustes > API > Claves/Aplicaciones > Añadir clave.</span>', 'woo_pagosnet_efectivo' )),
				),

				'key_cliente' => array(
					'title'       => __( 'Clave del cliente - API WooCommerce', 'woo_pagosnet_efectivo' ),
					'type'        => 'text',
					'description' => __( 'Genere esta clave en WooCommerce > Ajustes > API > Claves/Aplicaciones > Añadir clave' ),
					'default'     => '',
					'desc_tip'    => true,
				),

				'key_cliente_secreta' => array(
					'title'       => __( 'Clave secreta de cliente - API WooCommerce', 'woo_pagosnet_efectivo' ),
					'type'        => 'text',
					'description' => __( 'Genere esta clave en WooCommerce > Ajustes > API > Claves/Aplicaciones > Añadir clave' ),
					'default'     => '',
					'desc_tip'    => true,
				),
			) );
		}
		/**
		 * Mostrar las instrucciones (Código de referencia) en la página de pedido recibido.
		 */
		public function order_received_text( $text, $order ) {
		    if($order->payment_method == $this->id){
		    	return 'Su código de recaudo es: <span style="font-size: 20px; color: #333333;">'. get_post_meta( $order->get_order_number(),'id_transaccion', true ) . '</span> Realice su Pago en cualquiera de los puntos de PagosNet.';
		    }
		}

		/**
		 * Agregar contenido a  WC Emails.
		 *
		 * @access public
		 * @param WC_Order $order
		 * @param bool $sent_to_admin
		 * @param bool $plain_text
		 */
		public function email_instructions( $order, $sent_to_admin, $plain_text = false ) {
			if ( $this->idTransaccion && ! $sent_to_admin && $this->id === $order->payment_method && $order->has_status( 'on-hold' ) ) {
				echo wpautop( wptexturize('Su código de recaudo es: <span style="font-size: 20px; color: #333333;">'. $this->pagosNet->codigoRecaudacion . '</span> Realice su Pago en cualquiera de los puntos de PagosNet.') ) . PHP_EOL;
			}
		}
	
	
		/**
		 * Procesar el pago y retornar un resultado
		 *
		 * @param int $order_id
		 * @return array
		 */
		public function process_payment( $order_id ) {
			
			$this->pagosNet = new PagosNet($this->usuario, $this->pass, $this->url_pagosnet);

			$order = wc_get_order( $order_id );

			$this->pagosNet->correoElectronico = $order->data['billing']['email'];
			$this->pagosNet->codigoRecaudacion =  $order_id . date("dis");


			$this->pagosNet->nombreComprador = $order->data['billing']['first_name'] . ' ' . $order->data['billing']['last_name'];
			$this->pagosNet->montoPago = $order->data['total'];
			$this->pagosNet->montoCreditoFiscal = $order->data['total'];

			$idTransaccion = $this->pagosNet->registrarRecaudacion();

			
            
    		//El pago queda en espera
			if($idTransaccion){
				$order->update_status( 'pending', __( 'En espera de pago', 'woo_pagosnet_efectivo' ) );
				//Guardamos el código de recaudo
				add_post_meta($order_id, 'id_transaccion', $this->pagosNet->codigoRecaudacion, true);
			}else{
				$order->update_status( 'failed', __( 'Fallo el pago', 'woo_pagosnet_efectivo' ) );
			}

    		// Remove cart
 		    WC()->cart->empty_cart(); 	

    		 // Return thankyou redirect
		    return array(
		        'result'    => 'success',
		        'redirect'  => $this->get_return_url( $order )
		    );


		}
	
  } 
}


function woo_pagosnet_tarjeta_gateway_init() {

	class Woo_Pagosnet_tarjeta extends WC_Payment_Gateway {

		public $pagosNet = null;
		public $codigoRecaudacion = null;

		/**
		 * Constructor for the gateway.
		 */
		public function __construct() {
	  
			$this->id                 = 'woo_pagosnet_tarjeta';
			$this->icon               = plugins_url( 'woo-pagosnet/assets/img/logo.png', plugin_dir_path( __FILE__ ) );
			$this->has_fields         = false;
			$this->method_title       = __( 'PagosNet con Tarjeta de Crédito', 'woo_pagosnet_tarjeta' );
			$this->method_description = __( 'Realizar pagos con Tarjeta de Crédito usando PagosNet', 'woo_pagosnet_tarjeta' );
		  
			// Cargar configuraciones.
			$this->init_form_fields();
			$this->init_settings();

		  
			// Definir variables
			$this->title        = $this->get_option( 'title' );
			$this->description  = $this->get_option( 'description' );
			$this->usuario = $this->get_option( 'usuario');
			$this->pass = $this->get_option( 'pass');
			$this->url_pagosnet = $this->get_option( 'url_pagosnet');
			$this->codigo_empresa = $this->get_option( 'codigo_empresa');
			$this->select_url_frame = $this->get_option( 'select_url_frame');

		  
			// Actions
			add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, array( $this, 'process_admin_options' ) );
		  
			//Custom action 
			add_action( 'check_pagosnet', array( $this, 'check_response') );

			//Texto de la orden
			add_filter('woocommerce_thankyou_order_received_text', array($this, 'order_received_text'), 10, 2 );

			//Titulo
			add_filter( 'the_title', array($this, 'woo_title_order_received'), 10, 2 );
		
		}
	
		/**
		 * Initialize Gateway Settings Form Fields
		 */
		public function init_form_fields() {
	  
			$this->form_fields = apply_filters( 'wc_offline_form_fields', array(

				'url-api' => array(
					'title'       => __( 'Proporcione esta información a PagosNet para notificar los pagos en Efectivo', 'woo_pagosnet_tarjeta' ),
					'type'        => 'title',
					'description' => sprintf( __( '<code>'. plugins_url() . '/woo-pagosnet/api/public/registrar/transaccion' . '</code>', 'woo_pagosnet_tarjeta' )),
				),

		  
				'enabled' => array(
					'title'   => __( 'Activar/Desactivar', 'woo_pagosnet_tarjeta' ),
					'type'    => 'checkbox',
					'label'   => __( 'Activar los pagos en efectivo', 'woo_pagosnet_tarjeta' ),
					'default' => 'yes'
				),
				
				'title' => array(
					'title'       => __( 'Título', 'woo_pagosnet_tarjeta' ),
					'type'        => 'text',
					'description' => __( 'Título a mostrar' ),
					'default'     => __( 'Pagos en efectivo con PagosNet', 'woo_pagosnet_tarjeta' ),
					'desc_tip'    => true,
				),
				'usuario' => array(
					'title'       => __( 'Usuario WS PagosNet', 'woo_pagosnet_tarjeta' ),
					'type'        => 'text',
					'description' => __( 'Usuario proporcionado por PagosNet para el comercio' ),
					'default'     => '',
					'desc_tip'    => true,
				),
				'pass' => array(
					'title'       => __( 'Contraseña WS PagosNet', 'woo_pagosnet_tarjeta' ),
					'type'        => 'text',
					'description' => __( 'Contraseña proporcionada por PagosNet para el comercio' ),
					'default'     => __( 'Contraseña', 'woo_pagosnet_tarjeta' ),
					'desc_tip'    => true,
				),
				'url_pagosnet' => array(
					'title'       => __( 'URL de preproducción/producción', 'woo_pagosnet_tarjeta' ),
					'type'        => 'text',
					'description' => __( 'URL proporcionada por PagosNet para modo pruebas o producción.' ),
					'default'     => __( '', 'woo_pagosnet_tarjeta' ),
					'desc_tip'    => true,
				),
				'select_url_frame' => array(
					'title'       => __( 'URL Pagos con Tarjeta', 'woo_pagosnet_tarjeta' ),
					'type'        => 'text',
					'description' => __( 'URL proporcionada por PagosNet' ),
					'default'     => __( '', 'woo_pagosnet_tarjeta' ),
					'desc_tip'    => true,
					
				),
				'codigo_empresa' => array(
					'title'       => __( 'Código Empresa/Comercio', 'woo_pagosnet_tarjeta' ),
					'type'        => 'text',
					'description' => __( 'Código de empresa/comercio proporcionado por PagosNet.' ),
					'default'     => __( '', 'woo_pagosnet_tarjeta' ),
					'desc_tip'    => true,
				),
				'description' => array(
					'title'       => __( 'Descripción', 'woo_pagosnet_tarjeta' ),
					'type'        => 'text',
					'description' => __( 'Descripción para el método de pago' ),
					'default'     => __( 'Realice sus Pagos con Tarjeta de Crédito mediante PagosNet', 'woo_pagosnet_tarjeta' ),
					'desc_tip'    => true,
				),
			) );
		}
		
	
		/**
		 * Procesar el pago y retornar un resultado
		 *
		 * @param int $order_id
		 * @return array
		 */
		public function process_payment( $order_id ) {
			
			
			$this->pagosNet = new PagosNet($this->usuario, $this->pass, $this->url_pagosnet);

			$order = wc_get_order( $order_id );

			$this->pagosNet->correoElectronico = $order->data['billing']['email'];
			$this->pagosNet->codigoRecaudacion =  $order_id . date("dis");
			$this->pagosNet->nombreComprador = $order->data['billing']['first_name'] . ' ' . $order->data['billing']['last_name'];
			$this->pagosNet->nombre = $order->data['billing']['first_name'];
			$this->pagosNet->apellido = $order->data['billing']['last_name'];
			$this->pagosNet->montoPago = $order->data['total'];
			$this->pagosNet->montoCreditoFiscal = $order->data['total'];
			$this->pagosNet->telefono = $order->data['billing']['phone'];


			$idTransaccion = $this->pagosNet->registrarRecaudacion();
			$baseUrl = $this->get_return_url( $order );
			$urlResponse;  
			$tempUrl = str_replace('#', 'numero', $this->select_url_frame);
			
           

			if($idTransaccion){

				if($this->pagosNet->registrarTarjeta($idTransaccion) == 0){
					
					return array(
		        	'result'    => 'success',                                               
		        	'redirect'  => plugins_url() . '/woo-pagosnet/pago.php?entidad=' . $this->codigo_empresa . '&ref=' . $idTransaccion . '&url=' . $tempUrl . '&red=' . $baseUrl 
		    		);

				}

			}else{
				wc_add_notice(  $ex->getMessage(), 'error' );
				return array(
					'result' => 'failure',
					'redirect' => ''
				);
			}
		}


		/**
		 * Mostrar las instrucciones (Código de referencia) en la página de pedido recibido.
		 *
		 * @param String text
		 * @param Wc_Order order
		 * @return String
		 */
		public function order_received_text( $text, $order ) {
		    if($order->payment_method == $this->id){
		    	
		    	if(isset($_GET['success'])){

			    	if($_GET['success'] == 'false'){
			    		return '<span style="font-size: 20px; color: #333333;">No se ha podido procesar su pago. Si continua presentando problemas, Por favor contacté con el administrador del sitio.</span>';
			    	}else{
			    		return '<span style="font-size: 20px; color: #333333;">Su pago se ha procesado con éxito.</span>';
			    	}

		    	}else{
		    		return '<span style="font-size: 20px; color: #333333;">Pago cancelado.</span>';
		    	}
		    	    	
		    }
		}


		/**
		* Personalizar el titulo
		* @param String title
		* @param int id  
		* @return String
		*/
		public function woo_title_order_received( $title, $id ) {
			if ( function_exists( 'is_order_received_page' ) && 
			    is_order_received_page() && get_the_ID() === $id) {		

				if(isset($_GET['success'])){
					if($_GET['success'] == 'false'){
						$title = "Pago cancelado";
					}
				}else{
					$title = "Pago cancelado";
				}

			}

			return $title;
		}


		/**
		*Comprobar la respuesta del pago
		*/
		public function check_response(){

			if(isset($_GET['success'])){

				$status = $_GET['success'];
				$order_id = wc_get_order_id_by_order_key($_GET['key']);
 				$order = new WC_Order( $order_id );


				if( $order_id == 0 || $order_id == '' ) {
					return;
				}

				if( $order->has_status('completed') || $order->has_status('processing')) {
					return;
				}

				if($status == 'true'){
					// Payment complete
			  		$order->payment_complete( $order_id );
				}else{
		  			$order->update_status('cancelled', sprintf( __( '%s Pago Cancelado!', 'woocommerce' ), $this->title) );
				}

			}else{
				$order->update_status('cancelled', sprintf( __( '%s Pago Cancelado!', 'woocommerce' ), $this->title) );
			}


			// Remove cart
 		    WC()->cart->empty_cart(); 	

		
		}


	
  } 

}

	add_action( 'init', 'check_for_pagosnet' );
	function check_for_pagosnet() {
		if( isset($_GET['success'])) {
		  // Start the gateways
			WC()->payment_gateways();
			do_action( 'check_pagosnet' );
		}
	}
